//*****************************************************************************
//*
//* FILE NAME:  RACFResourceAccess.java
//*
//* IBM Confidential
//*
//*  OCO Source Materials
//*
//*  5647-A01
//*
//*  (C) Copyright IBM Corp. 2000
//*
//*   The source code for this program is not published or otherwise
//*   divested of its trade secrets, irrespective of what has been
//*   deposited with the U.S. Copyright Office.
//*
//*   Status = JMSI619
//*
//*****************************************************************************
//*
//* COMPONENT:  Guenter Dockweiler
//*
//* CLASSES:  RACFResourceAccess (externally documented: no)
//*
//* CHANGE HISTORY:
//*
//*   Change Date     UserID   Description
//*   -----  -------- -------- ------------------------------------------------
//*   c0001  07/01/99 DOCK     initial version
//*   c0002  01/01/00 schawer  changes for simulation
//*
//*****************************************************************************
package com.ibm.irr.bind;

import com.ibm.cim.common.*;
import com.ibm.cim.util.*;

/** RACFResourceAccess implements the low-level interface to RACF
  * resources by issueing RACF commands. This is implemented by a native C
  * method which in turn calls a PL/X module to talk to RACF.
*/
/** @author G. Dockweiler */

class RACFResourceAccess
{
  private static String    ivCopyright = RACFCopyright.IBM_2000;
  private static String    CLASS       = "RACFResourceAccess";

  private static CimExtendedTrace cvTrace = RACFResourceController.getExtendedTrace();

  /**@shapeType DependencyLink
  * @label throws*/
  /*#  RACFInternalError lnkUnnamed1*/
  // -------------------------------------------------------------------
  // static code to load library containing native C code
  // -------------------------------------------------------------------
/* [c0002] -ws
  static
  {
    System.loadLibrary("irryciml");
  }
*/

  // -------------------------------------------------------------------
  // instance variables
  // -------------------------------------------------------------------
  int ivResultSAFRC;
  int ivResultRACFRC;
  int ivResultReasonCode;
  int ivResultNative;

  String ivUserID;

  // -------------------------------------------------------------------
  // constructor
  // -------------------------------------------------------------------
  public RACFResourceAccess(String userID)
  {
    String METHOD = "RACFResourceAccess()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("User ID: "+userID,true,CLASS,METHOD); }

    ivUserID = userID;

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall(null,false,CLASS,METHOD); }
  }

  /**Execute RACF command by calling native method and actualize command output object
   * @param command is the RACF command to be executed
   * @param commandOutput is the command output object that is actualized after command execution
   * @return true if the command completed successfully, otherwise false
   * @exception RACFInternalError
  */
  // -------------------------------------------------------------------
  // perform basic success/no success check of command base on return
  // info of IRRSEQ00 RACF callable service:
  //
  //  SAF|RACF|Reason|                   |                   |return
  //  rc |rc  |Code  |Meaning            |Exception thrown   |value
  // ----+----+------+-------------------+-------------------+-------
  //   0 |  0 |  0   |Everything ok      |-                  |true
  //   4 |  0 |  0   |RACF not installed |- *)               |false
  //   8 |  8 |  0   |Incorrect IRRSEQ00 |RACF_INTERNAL_ERROR|false
  //     |    |      |function code      |                   |
  //   8 |  8 |  4   |IRRSEQ00 input parm|RACF_INTERNAL_ERROR|false
  //     |    |      |list error         |                   |
  //   8 |  8 |  8   |Invalid IRRSEQ00   |RACF_INTERNAL_ERROR|false
  //     |    |      |input parmlist data|                   |
  //   8 |  8 | 12   |Unable to establish|RACF_INTERNAL_ERROR|false
  //     |    |      |recovery environm  |                   |
  //   8 |  8 | 16   |Invalid command    |RACF_INTERNAL_ERROR|false
  //     |    |      |passed to IRRSEQ00 |                   |
  //   8 | 12 | any  |Unable to invoke   |- *)               |false
  //     |    |      |RACF               |                   |
  //   8 | 16 |  4 +)|-                  |- *)               |false
  //     |    |      | |RACF command     |                   |
  //   8 | 16 |  8 +)| |error            |- *)               |false
  //     |    |      | |(may or may not  |                   |
  //   8 | 16 | 12 +)| | be ok)          |- *)               |false
  //     |    |      |-                  |                   |
  //     |    | >500 |e.g. invalid userID|RACF_INTERNAL_ERROR|false
  // ----+----+------+-------------------+-------------------+-------
  //                                      *) may be 'soft' error (handled separately)
  //           +) RACF command return code
  // -----------------------------------------------------------------
  public boolean issueCommand
   (String            command,
    RACFCommandOutput commandOutput)
   throws RACFInternalError
  {
    String METHOD = "issueCommand()";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Command: "+command,true,CLASS,METHOD); }

    String  commandResult  = new String();
    String  lastCommand    = new String();
    String  resultBuffer   = null;
    boolean result = true;

    ivResultSAFRC      = RACFConstants.RESULT_OK;
    ivResultRACFRC     = RACFConstants.RESULT_OK;
    ivResultReasonCode = RACFConstants.RESULT_OK;
    ivResultNative     = RACFConstants.RESULT_OK;

    // save last command, e.g. ADDUSER, DELGROUP, etc (first token of command)
    lastCommand = command.substring(0,command.indexOf(' '));
    commandResult = "";

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.resourceAccess(lastCommand,"Issue RACF command (native method call)"+
                                                                        cvTrace.getIndentString(),
                                                                        "Command: "+command+cvTrace.getIndentString()+
                                                                        "User ID: "+ivUserID); }
    resultBuffer = this.issueCommandNative(command,ivUserID);
    if (null == resultBuffer) ivResultNative = RACFConstants.RESULT_JNI_PROBLEM;
    commandResult = resultBuffer;

    // actualize the command result object
    commandOutput.setResult(ivResultSAFRC,
                            ivResultRACFRC,
                            ivResultReasonCode,
                            ivResultNative,
                            lastCommand,
                            commandResult);

    // any problem?            -> command resuld is 'false'
    // any unexpected problem? -> throw exception
    if (RACFConstants.RESULT_OK < ivResultNative) // this may be - COMMAND_PROBLEM
    {                                             //             - JNI_PROBLEM
                                                  //             - STORAGE_PROBLEM
                                                  //             - OUTPUT_TRUNC
      result = false;                             // -> unsuccessful result

      if (RACFConstants.RESULT_JNI_PROBLEM      == ivResultNative ||
          RACFConstants.RESULT_STORAGE_PROBLEM  == ivResultNative ||
          RACFConstants.RESULT_OUTPUT_TRUNC     == ivResultNative ||
          (RACFConstants.RESULT_COMMAND_PROBLEM == ivResultNative &&
           500 <= ivResultReasonCode))   // e.g. invalid user ID

      {
        // JNI_PROBLEM                            -> exception
        // STORAGE_PROBLEM                        -> exception
        // OUTPUT_TRUNC                           -> exception
        // COMMAND_PROBLEM and reason code >= 500 -> exception

        if (CimExtendedTrace.cvTrcLvl) { cvTrace.diagnostic("Throwing RACFInternalError exception"); }
        throw new RACFInternalError(commandOutput,
                                    command);
      }

    }

  //System.out.println("C method result: " + ivResultNative);
  //System.out.println("SAFRC:           " + ivResultSAFRC);
  //System.out.println("RACFRC:          " + ivResultRACFRC);
  //System.out.println("Reason code:     " + ivResultReasonCode);
  //System.out.println("Command output:");
  //System.out.println(commandResult);

    if (CimExtendedTrace.cvTrcLvl) { cvTrace.externalCall("Command result: "+result,true,CLASS,METHOD); }

    return result;

  }

  /**Native method to issue RACF command
  */
  // -------------------------------------------------------------------
  // the called C routine
  //  - submits command that is passed as input parm
  //  - sets variables resultSAFRC, resultRACFRC, resultReasonCode and resultNative
  //  - instantiates String resultBuffer as return value
  //    (lines in result buffer are separated by '\n' characters)
  // -------------------------------------------------------------------

  // [c0002] -ws private native String issueCommandNative (String command, String userID);
  private String issueCommandNative (String command, String userID) {
     return "IRR0000 -ws simulated!";
  }

}
